@@ -511,8 +511,20 @@ class DirectoryBasedExampleDatabase(ExampleDatabase):
     def delete(self, key: bytes, value: bytes) -> None:
         try:
             self._value_path(key, value).unlink()
+        except OSError:
+            return
+
+        # try deleting the key dir, which will only succeed if the dir is empty
+        # (i.e. ``value`` was the last value in this key).
+        try:
+            self._key_path(key).rmdir()
         except OSError:
             pass
+        else:
+            # if the deletion succeeded, also delete this key entry from metakeys.
+            # (if this key happens to be the metakey itself, this deletion will
+            # fail; that's ok and faster than checking for this rare case.)
+            self.delete(self._metakeys_name, key)
 
     def _start_listening(self) -> None:
         try:
